// impulse.test.js

if (typeof expect === "undefined") { global.expect = require("expect.js"); }
if (typeof Arsenal === "undefined") { global.Arsenal = require("../arsenal.js"); }

// TEST CASE: Impulse mechanism
describe("Impulse", function () {

    this.timeout(0);

    before(function () {

        this.Impulse = Arsenal.Impulse;

    });

    it("initialize successfully", function () {

        expect(this.Impulse).to.be.ok();

        var impulse = new this.Impulse();

        expect(impulse).to.be.an(Object);

        expect(impulse.addWatcher).to.be.a(Function);
        expect(impulse.getData).to.be.a(Function);
        expect(impulse.setData).to.be.a(Function);

        expect(impulse.on).to.be(impulse.addWatcher);
        expect(impulse.get).to.be(impulse.getData);
        expect(impulse.set).to.be(impulse.setData);

    });

    describe("Test.1", function () {

        before(function () {

            this.data = {
                a: {
                    x: 0
                },
                b: {
                    x: 0,
                    y: 1
                },
                c: {
                    x: 0,
                    y: 1
                }
            };

            this.impulse = new this.Impulse(this.data);

        });

        it("watch on single data-node", function () {

            this.impulse.on("a.x", function (x) {
                expect(x).to.be(2);
            });

            this.impulse.set("a.x", 2);

        });

        it("watch on multiple data-nodes", function () {

            var index = 0;

            this.impulse.on([
                "b.x",
                "b.y"
            ], function (x, y) {
                switch (++index) {
                    case 1:
                        expect(x).to.be(3);
                        expect(y).to.be(1);
                    break;

                    case 2:
                        expect(x).to.be(3);
                        expect(y).to.be(4);
                    break;
                }
            });

            this.impulse.set("b.x", 3);
            this.impulse.set("b.y", 4);

        });

        it("watch with conditional functions", function () {

            var counter = 0;

            this.impulse.on({
                "c.x": function (x) {
                    return (x === 3);
                },
                "c.y": function (y) {
                    return (y === 4);
                }
            }, function (x, y) {
                expect(x).to.be(3);
                expect(y).to.be(4);
                counter++;
            });

            this.impulse.set("c.x", 3);
            expect(counter).to.be(0);
            this.impulse.set("c.y", 4);
            expect(counter).to.be(1);
            this.impulse.set("c.x", 0);
            expect(counter).to.be(1);
            this.impulse.set("c.y", 1);
            expect(counter).to.be(1);

        });

        it("watch with conditional values", function () {

            var counter = 0;

            this.impulse.on({
                "c.x": 31,
                "c.y": 47
            }, function (x, y) {
                expect(x).to.be(31);
                expect(y).to.be(47);
                counter++;
            });

            this.impulse.set("c.x", 31);
            expect(counter).to.be(0);
            this.impulse.set("c.y", 47);
            expect(counter).to.be(1);
            this.impulse.set("c.x", 0);
            expect(counter).to.be(1);
            this.impulse.set("c.y", 1);
            expect(counter).to.be(1);

        });

    });

});
