// pipeline.test.js

if (typeof expect === "undefined") { global.expect = require("expect.js"); }
if (typeof Arsenal === "undefined") { global.Arsenal = require("../arsenal.js"); }

// TEST CASE: Pipeline mechanism
describe("Pipeline", function () {

    this.timeout(0);

    before(function () {

        this.Pipeline = Arsenal.Pipeline;

    });

    it("initialized successfully", function () {

        expect(this.Pipeline).to.be.ok();

        var pipeline = new this.Pipeline();

        expect(pipeline).to.be.an(Object);

        expect(Arsenal.Pipeline.createFlow).to.be.a(Function);

        expect(pipeline.create).to.be.a(Function);
        expect(pipeline.load).to.be.a(Function);
        expect(pipeline.unload).to.be.a(Function);
        expect(pipeline.clear).to.be.a(Function);
        expect(pipeline.check).to.be.a(Function);
        expect(pipeline.get).to.be.a(Function);
        expect(pipeline.run).to.be.a(Function);

    });

    describe("Test.1", function () {

        it("consistent pipeline for serial flows", function (done) {

            var pipeline = new this.Pipeline(),
                createFlow = Arsenal.Pipeline.createFlow,
                startValue = 1,
                resultValue = 6,
                testFlow1 = createFlow("Test flow 1", function (F) {

                    F("entry of flow 1", function (a) {
                        this.a = a;
                        return F("step2", [ a ]);
                    });

                    F("step2", function (b) {
                        this.b = this.a + b;
                        return F("Test flow 2", [ this.b ], F("feedback"));
                    });

                    F("feedback", function (result) {
                        return result;
                    });

                }),
                testFlow2 = createFlow("Test flow 2", function (F) {

                    F("entry of flow 2", function (b) {
                        this.c = this.a + b;
                        return F("calculate");
                    });

                    F("calculate", function () {
                        return this.a + this.b + this.c;
                    });

                });

            pipeline.load(testFlow1);
            pipeline.load(testFlow2);

            expect(pipeline.check("Test flow 1")).to.be(true);
            expect(pipeline.check("Test flow 2")).to.be(true);

            pipeline.run("Test flow 1", [ startValue ], {}, function (result) {
                expect(result).to.be(resultValue);
                done();
            });

            // Dummy
            expect(0).to.be(0);

            this.timeout(100);

        });

    });

    describe("Test.2", function () {

        it("inconsistent pipeline for parallel flows", function (done) {

            var pipeline = new this.Pipeline({
                    consistent: false
                }),
                createFlow = Arsenal.Pipeline.createFlow,
                startValue = 1,
                resultValue = 6,
                mainFlow = createFlow("Main flow", function (F) {

                    F("entry of main flow", function (value) {
                        this.totalFlow = 2;
                        this.doneFlow = 0;
                        F("Test flow 1", [ value ], F("flow 1 feedback"))();
                        F("Test flow 2", [ value ], F("flow 2 feedback"))();
                    });

                    F("flow 1 feedback", function (result) {
                        this.result1 = result;
                        F("settle")();
                    });

                    F("flow 2 feedback", function (result) {
                        this.result2 = result;
                        F("settle")();
                    });

                    F("settle", function () {
                        if (++this.doneFlow === this.totalFlow) {
                            return this.result1 + this.result2;
                        }
                    });

                }),
                testFlow1 = createFlow("Test flow 1", function (F) {

                    F("entry of flow 1", function (a) {
                        this.a = a;
                        return F("step2", [ a ]);
                    });

                    F("step2", function (b) {
                        this.b = this.a + b;
                        return F("Test flow 2", [ this.b ], F("feedback"));
                    });

                    F("feedback", function (result) {
                        return result;
                    });

                }),
                testFlow2 = createFlow("Test flow 2", function (F) {

                    F("entry of flow 2", function (b) {
                        this.b = b;
                        return F("calculate");
                    });

                    F("calculate", function () {
                        return this.b * 2;
                    });

                });

            pipeline.load(mainFlow);
            pipeline.load(testFlow1);
            pipeline.load(testFlow2);

            expect(pipeline.check("Main flow")).to.be(true);
            expect(pipeline.check("Test flow 1")).to.be(true);
            expect(pipeline.check("Test flow 2")).to.be(true);

            pipeline.run("Main flow", [ startValue ], {}, function (result) {
                expect(result).to.be(resultValue);
                done();
            });

            // Dummy
            expect(0).to.be(0);

            this.timeout(100);

        });

    });

});
